@@ -1,14 +1,17 @@
 from __future__ import absolute_import, unicode_literals
 
 import socket
+
 import pytest
-from case import patch, call, Mock, ANY
-from amqp import spec, Connection, Channel, sasl, Message
+
+import amqp
+from amqp import Channel, Connection, Message, sasl, spec
+from amqp.exceptions import (AccessRefused, ConnectionError, InvalidCommand,
+                             NotFound, PreconditionFailed, ResourceLocked)
 from amqp.platform import pack
-from amqp.exceptions import ConnectionError, \
-    InvalidCommand, AccessRefused, PreconditionFailed, NotFound, ResourceLocked
-from amqp.serialization import dumps, loads
 from amqp.protocol import queue_declare_ok_t
+from amqp.serialization import dumps, loads
+from case import ANY, Mock, call, patch
 
 connection_testdata = (
     (spec.Connection.Blocked, '_on_blocked'),
@@ -60,17 +63,17 @@ queue_declare_error_testdata = (
     ),
 )
 
-CLIENT_CAPABILITIES = {
+CLIENT_PROPERTIES = {
     'product': 'py-amqp',
-    'product_version': '2.3.2',
+    'product_version': amqp.__version__,
     'capabilities': {
         'consumer_cancel_notify': True,
         'connection.blocked': True,
         'authentication_failure_close': True
-    }
+    },
 }
 
-SERVER_CAPABILITIES = {
+SERVER_PROPERTIES = {
     'capabilities': {
         'publisher_confirms': True,
         'exchange_exchange_bindings': True,
@@ -124,13 +127,16 @@ class DataComparator(object):
         return tuple(values) == tuple(self.items)
 
 
-def handshake(conn, transport_mock):
+def handshake(conn, transport_mock, server_properties=None):
     # Helper function simulating connection handshake with server
+    if server_properties is None:
+        server_properties = SERVER_PROPERTIES
+
     transport_mock().read_frame.side_effect = [
         build_frame_type_1(
             spec.Connection.Start, channel=0,
             args=(
-                0, 9, SERVER_CAPABILITIES, 'AMQPLAIN PLAIN', 'en_US'
+                0, 9, server_properties, 'AMQPLAIN PLAIN', 'en_US'
             ),
             arg_format='ooFSS'
         ),
@@ -193,7 +199,7 @@ class test_connection:
                         DataComparator(
                             'FsSs',
                             (
-                                CLIENT_CAPABILITIES, 'AMQPLAIN',
+                                CLIENT_PROPERTIES, 'AMQPLAIN',
                                 security_mechanism,
                                 'en_US'
                             )
@@ -215,6 +221,131 @@ class test_connection:
                     )
                 ]
             )
+            assert conn.client_properties == CLIENT_PROPERTIES
+
+    def test_connect_no_capabilities(self):
+        # Test checking connection handshake with broker
+        # not supporting capabilities
+        frame_writer_cls_mock = Mock()
+        on_open_mock = Mock()
+        frame_writer_mock = frame_writer_cls_mock()
+        conn = Connection(
+            frame_writer=frame_writer_cls_mock, on_open=on_open_mock
+        )
+
+        with patch.object(conn, 'Transport') as transport_mock:
+            server_properties = dict(SERVER_PROPERTIES)
+            del server_properties['capabilities']
+            client_properties = dict(CLIENT_PROPERTIES)
+            del client_properties['capabilities']
+
+            handshake(
+                conn, transport_mock, server_properties=server_properties
+            )
+            on_open_mock.assert_called_once_with(conn)
+            security_mechanism = sasl.AMQPLAIN(
+                'guest', 'guest'
+            ).start(conn).decode('utf-8', 'surrogatepass')
+
+            # Expected responses from client
+            frame_writer_mock.assert_has_calls(
+                [
+                    call(
+                        1, 0, spec.Connection.StartOk,
+                        # Due Table type, we cannot compare bytestream directly
+                        DataComparator(
+                            'FsSs',
+                            (
+                                client_properties, 'AMQPLAIN',
+                                security_mechanism,
+                                'en_US'
+                            )
+                        ),
+                        None
+                    ),
+                    call(
+                        1, 0, spec.Connection.TuneOk,
+                        dumps(
+                            'BlB',
+                            (conn.channel_max, conn.frame_max, conn.heartbeat)
+                        ),
+                        None
+                    ),
+                    call(
+                        1, 0, spec.Connection.Open,
+                        dumps('ssb', (conn.virtual_host, '', False)),
+                        None
+                    )
+                ]
+            )
+            assert conn.client_properties == client_properties
+
+    def test_connect_missing_capabilities(self):
+        # Test checking connection handshake with broker
+        # supporting subset of capabilities
+        frame_writer_cls_mock = Mock()
+        on_open_mock = Mock()
+        frame_writer_mock = frame_writer_cls_mock()
+        conn = Connection(
+            frame_writer=frame_writer_cls_mock, on_open=on_open_mock
+        )
+
+        with patch.object(conn, 'Transport') as transport_mock:
+            server_properties = dict(SERVER_PROPERTIES)
+            server_properties['capabilities'] = {
+                # This capability is not supported by client
+                'basic.nack': True,
+                'consumer_cancel_notify': True,
+                'connection.blocked': False,
+                # server does not support 'authentication_failure_close'
+                # which is supported by client
+            }
+
+            client_properties = dict(CLIENT_PROPERTIES)
+            client_properties['capabilities'] = {
+                'consumer_cancel_notify': True,
+            }
+
+            handshake(
+                conn, transport_mock, server_properties=server_properties
+            )
+            on_open_mock.assert_called_once_with(conn)
+            security_mechanism = sasl.AMQPLAIN(
+                'guest', 'guest'
+            ).start(conn).decode('utf-8', 'surrogatepass')
+
+            # Expected responses from client
+            frame_writer_mock.assert_has_calls(
+                [
+                    call(
+                        1, 0, spec.Connection.StartOk,
+                        # Due Table type, we cannot compare bytestream directly
+                        DataComparator(
+                            'FsSs',
+                            (
+                                client_properties, 'AMQPLAIN',
+                                security_mechanism,
+                                'en_US'
+                            )
+                        ),
+                        None
+                    ),
+                    call(
+                        1, 0, spec.Connection.TuneOk,
+                        dumps(
+                            'BlB',
+                            (conn.channel_max, conn.frame_max, conn.heartbeat)
+                        ),
+                        None
+                    ),
+                    call(
+                        1, 0, spec.Connection.Open,
+                        dumps('ssb', (conn.virtual_host, '', False)),
+                        None
+                    )
+                ]
+            )
+            assert conn.client_properties == client_properties
 
     def test_connection_close(self):
         # Test checking closing connection
@@ -414,7 +545,7 @@ class test_channel:
             )
             frame_writer_mock = frame_writer_cls_mock()
             frame_writer_mock.reset_mock()
-            ch.basic_consume('my_queue', callback=callback_mock)
+            ret = ch.basic_consume('my_queue', callback=callback_mock)
             frame_writer_mock.assert_called_once_with(
                 1, 1, spec.Basic.Consume,
                 dumps(
@@ -424,6 +555,7 @@ class test_channel:
                 None
             )
             assert ch.callbacks[consumer_tag] == callback_mock
+            assert ret == 'amq.ctag-PCmzXGkhCw_v0Zq7jXyvkg'
 
     def test_consume_with_consumer_tag(self):
         # Test verifing starting consuming with specified consumer_tag
@@ -443,7 +575,7 @@ class test_channel:
             )
             frame_writer_mock = frame_writer_cls_mock()
             frame_writer_mock.reset_mock()
-            ch.basic_consume(
+            ret = ch.basic_consume(
                 'my_queue', callback=callback_mock, consumer_tag='my_tag'
             )
             frame_writer_mock.assert_called_once_with(
@@ -458,6 +590,7 @@ class test_channel:
                 None
             )
             assert ch.callbacks['my_tag'] == callback_mock
+            assert ret == 'my_tag'
 
     def test_queue_declare(self):
         # Test verifying declaring queue
