@@ -1,16 +1,18 @@
 from __future__ import absolute_import, unicode_literals
 
-import pytest
 import socket
-from case import ContextMock, Mock, patch, ANY, MagicMock
+
+import pytest
+from vine import promise
 
 from amqp import spec
 from amqp.basic_message import Message
+from amqp.channel import Channel
+from amqp.exceptions import (ConsumerCancelled, MessageNacked, NotFound,
+                             RecoverableConnectionError)
 from amqp.platform import pack
 from amqp.serialization import dumps
-from amqp.channel import Channel
-from amqp.exceptions import ConsumerCancelled, NotFound, MessageNacked, \
-    RecoverableConnectionError
+from case import ANY, ContextMock, MagicMock, Mock, patch
 
 
 class test_Channel:
@@ -293,7 +295,7 @@ class test_Channel:
     def test_basic_consume_no_consumer_tag(self):
         callback = Mock()
         self.c.send_method.return_value = (123,)
-        self.c.basic_consume(
+        ret = self.c.basic_consume(
             'q', arguments={'x': 1},
             callback=callback,
         )
@@ -304,10 +306,13 @@ class test_Channel:
             returns_tuple=True
         )
         assert self.c.callbacks[123] is callback
+        assert ret == 123
 
     def test_basic_consume_no_wait(self):
         callback = Mock()
-        self.c.basic_consume(
+        ret_promise = promise()
+        self.c.send_method.return_value = ret_promise
+        ret = self.c.basic_consume(
             'q', 123, arguments={'x': 1},
             callback=callback, nowait=True
         )
@@ -318,6 +323,7 @@ class test_Channel:
             returns_tuple=True
         )
         assert self.c.callbacks[123] is callback
+        assert ret == ret_promise
 
     def test_basic_consume_no_wait_no_consumer_tag(self):
         callback = Mock()
@@ -464,6 +470,31 @@ class test_Channel:
             (0, 'ex', 'rkey', False, False), 'msg',
         )
 
+    def test_basic_publish_connection_blocked_not_supported_missing(self):
+        # Test veryfying that when server does not have
+        # connection.blocked capability, drain_events() are not called
+        self.conn.client_properties = {
+            'capabilities': {}
+        }
+        self.c._basic_publish('msg', 'ex', 'rkey')
+        self.conn.drain_events.assert_not_called()
+        self.c.send_method.assert_called_once_with(
+            spec.Basic.Publish, 'Bssbb',
+            (0, 'ex', 'rkey', False, False), 'msg',
+        )
+
+    def test_basic_publish_connection_blocked_no_capabilities(self):
+        # Test veryfying that when server does not have
+        # support of capabilities, drain_events() are not called
+        self.conn.client_properties = {
+        }
+        self.c._basic_publish('msg', 'ex', 'rkey')
+        self.conn.drain_events.assert_not_called()
+        self.c.send_method.assert_called_once_with(
+            spec.Basic.Publish, 'Bssbb',
+            (0, 'ex', 'rkey', False, False), 'msg',
+        )
+
     def test_basic_publish_confirm_callback(self):
 
         def wait_nack(method, *args, **kwargs):
