@@ -2,14 +2,17 @@ from __future__ import absolute_import, unicode_literals
 
 import errno
 import socket
+import struct
 
 import pytest
-from case import ANY, Mock, call, patch
 
 from amqp import transport
 from amqp.exceptions import UnexpectedFrame
 from amqp.platform import pack
 from amqp.transport import _AbstractTransport
+from case import ANY, MagicMock, Mock, call, patch
+
+SIGNED_INT_MAX = 0x7FFFFFFF
 
 
 class DummyException(Exception):
@@ -211,15 +214,24 @@ class test_socket_options:
 
     def test_set_sockopt_opts_timeout(self):
         # tests socket options SO_RCVTIMEO and SO_SNDTIMEO
-        # this test is soley for coverage as socket.settimeout
-        # is pythonic way to have timeouts
         self.transp = transport.Transport(
             self.host, self.connect_timeout,
         )
-        self.transp.read_timeout = 0xdead
-        self.transp.write_timeout = 0xbeef
-        with patch('socket.socket', return_value=MockSocket()):
-            self.transp.connect()
+        read_timeout_sec, read_timeout_usec = 0xdead, 0xbeef
+        write_timeout_sec = 0x42
+
+        self.transp.read_timeout = read_timeout_sec + \
+            read_timeout_usec * 0.000001
+        self.transp.write_timeout = write_timeout_sec
+        self.transp.connect()
+
+        expected_rcvtimeo = struct.pack('ll', read_timeout_sec,
+                                        read_timeout_usec)
+        expected_sndtimeo = struct.pack('ll', write_timeout_sec, 0)
+        assert expected_rcvtimeo == self.socket.getsockopt(socket.SOL_TCP,
+                                                           socket.SO_RCVTIMEO)
+        assert expected_sndtimeo == self.socket.getsockopt(socket.SOL_TCP,
+                                                           socket.SO_SNDTIMEO)
 
 
 class test_AbstractTransport:
@@ -319,6 +331,15 @@ class test_AbstractTransport:
         with pytest.raises(UnexpectedFrame):
             self.t.read_frame()
 
+    def test_read_frame__long(self):
+        self.t._read = Mock()
+        self.t._read.side_effect = [pack('>BHI', 1, 1, SIGNED_INT_MAX + 16),
+                                    b'read1', b'read2', b'\xce']
+        frame_type, channel, payload = self.t.read_frame()
+        assert frame_type == 1
+        assert channel == 1
+        assert payload == b'read1read2'
+
     def transport_read_EOF(self):
         for host, ssl in (('localhost:5672', False),
                           ('localhost:5671', True),):
@@ -403,6 +424,24 @@ class test_AbstractTransport:
             sock_mock.gettimeout.assert_called()
             sock_mock.settimeout.assert_not_called()
 
+    def test_set_timeout_ewouldblock_exc(self):
+        # We expect EWOULDBLOCK to be handled as a timeout.
+        with patch.object(self.t, 'sock') as sock_mock:
+            sock_mock.gettimeout.return_value = 3
+            with pytest.raises(socket.timeout):
+                with self.t.having_timeout(5):
+                    err = socket.error()
+                    err.errno = errno.EWOULDBLOCK
+                    raise err
+
+            class DummySocketError(socket.error):
+                pass
+
+            # Other socket errors shouldn't be converted.
+            with pytest.raises(DummySocketError):
+                with self.t.having_timeout(5):
+                    raise DummySocketError()
+
 
 class test_AbstractTransport_connect:
 
@@ -583,11 +622,21 @@ class test_SSLTransport:
             assert ctx.check_hostname
             ctx.wrap_socket.assert_called_with(sock, f=1)
 
+    def test_wrap_socket_sni(self):
+        sock = Mock()
+        with patch('ssl.wrap_socket') as mock_ssl_wrap:
+            self.t._wrap_socket_sni(sock)
+            mock_ssl_wrap.assert_called_with(cert_reqs=0, certfile=None,
+                                             keyfile=None, sock=sock,
+                                             ca_certs=None, server_side=False,
+                                             ciphers=None, ssl_version=2,
+                                             suppress_ragged_eofs=True,
+                                             do_handshake_on_connect=True)
+
     def test_shutdown_transport(self):
         self.t.sock = None
         self.t._shutdown_transport()
-        self.t.sock = object()
-        self.t._shutdown_transport()
+
         sock = self.t.sock = Mock()
         self.t._shutdown_transport()
         assert self.t.sock is sock.unwrap()
@@ -600,6 +649,43 @@ class test_SSLTransport:
                            match=r'.*Server unexpectedly closed connection.*'):
             self.t._read(64)
 
+    def test_write_success(self):
+        self.t.sock = Mock(name='SSLSocket')
+        self.t.sock.write.return_value = 2
+        self.t._write('foo')
+        self.t.sock.write.assert_called()
+
+    def test_write_socket_closed(self):
+        self.t.sock = Mock(name='SSLSocket')
+        self.t.sock.write.return_value = ''
+        with pytest.raises(IOError,
+                           match=r'.*Socket closed.*'):
+            self.t._write('foo')
+
+    def test_write_ValueError(self):
+        self.t.sock = Mock(name='SSLSocket')
+        self.t.sock.write.return_value = 2
+        self.t.sock.write.side_effect = ValueError("Some error")
+        with pytest.raises(IOError,
+                           match=r'.*Socket closed.*'):
+            self.t._write('foo')
+
+    def test_read_timeout(self):
+        self.t.sock = Mock(name='SSLSocket')
+        self.t._quick_recv = Mock(name='recv', return_value='4')
+        self.t._quick_recv.side_effect = socket.timeout()
+        self.t._read_buffer = MagicMock(return_value='AA')
+        with pytest.raises(socket.timeout):
+            self.t._read(64)
+
+    def test_read_SSLError(self):
+        self.t.sock = Mock(name='SSLSocket')
+        self.t._quick_recv = Mock(name='recv', return_value='4')
+        self.t._quick_recv.side_effect = transport.SSLError('timed out')
+        self.t._read_buffer = MagicMock(return_value='AA')
+        with pytest.raises(socket.timeout):
+            self.t._read(64)
+
 
 class test_TCPTransport:
 
